/* mbed Microcontroller Library
 * Copyright (c) 2006-2013 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "mbed.h"
#include "BLE.h"
#include "DiscoveredCharacteristic.h"
#include "DiscoveredService.h"
#include "GattClient.h"

/**
 * - This program will read the strongest address
 * - Print the address
 * - Print the 128 bit UUID
 * - Invite a connect
 * - Request Service UUID's
 * - //Disconnect
 * - Set Characteristics
 */

BLE ble;
DiscoveredCharacteristic MYO1;
DigitalOut led1(LED1,1);

bool triggerLedCharacteristic = false;
int command =0;
char data_start[] = "0103020101";
uint64_t data_setup = 0x0102020301;
const int configure1 = 0x01;
const uint16_t configure2 = 0x02;
DiscoveredCharacteristic ledCharacteristic;

void periodicCallback(void)
{
    led1 = !led1; //Force a blinking light
    printf("command == %02d \r\n",command);
}

void advertisementCallback(const Gap::AdvertisementCallbackParams_t *params) {

    printf("Adv peerAddr: [%02x %02x %02x %02x %02x %02x] rssi %d, ScanResp: %u, AdvType: %u\r\n",
           params->peerAddr[5], params->peerAddr[4], params->peerAddr[3], params->peerAddr[2], params->peerAddr[1], params->peerAddr[0],
           params->rssi, params->isScanResponse, params->type);
    for (unsigned index = 0; index < params->advertisingDataLen; index++) {
        printf("%02x ", params->advertisingData[index]);
    }
    printf("\r\n");
    
    ble.gap().connect(params->peerAddr, Gap::ADDR_TYPE_PUBLIC, NULL, NULL);
}

void discoveryTerminationCallback(Gap::Handle_t connectionHandle) {
    printf("terminated SD for handle %u\r\n", connectionHandle);
    ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x19, 0x05, (reinterpret_cast<const uint8_t *>(&data_setup)));
    command = 1; // Loop Counter for temporary setup Que
}

void serviceDiscoveryCallback(const DiscoveredService *service) {
    if (service->getUUID().shortOrLong() == UUID::UUID_TYPE_SHORT) {
        printf("S UUID-%x attrs[%x %x]\r\n", service->getUUID().getShortUUID(), service->getStartHandle(), service->getEndHandle());
    } else {
        printf("S UUID-");
        const uint8_t *longUUIDBytes = service->getUUID().getBaseUUID();
        for (unsigned i = 0; i < UUID::LENGTH_OF_LONG_UUID; i++) {
            printf("%02x", longUUIDBytes[i]);
        }
        printf(" attrs[%x %x]\r\n", service->getStartHandle(), service->getEndHandle());
    }
}

void characteristicDiscoveryCallback(const DiscoveredCharacteristic *characteristicP) {
    printf("  C UUID-%x valueAttr[%x] props[%x]\r\n", characteristicP->getShortUUID(), characteristicP->getValueHandle(), 
    (uint8_t)characteristicP->getProperties().broadcast());
    if (characteristicP->getShortUUID() != 0xa001) { /* !ALERT! Alter this filter to suit your device. */
        ledCharacteristic        = *characteristicP;
        triggerLedCharacteristic = true;
    }
}

void connectionCallback(const Gap::ConnectionCallbackParams_t *params) {
    if (params->role == Gap::CENTRAL) {
        ble.gattClient().onServiceDiscoveryTermination(discoveryTerminationCallback);
        ble.gattClient().launchServiceDiscovery(params->handle, serviceDiscoveryCallback, 
        characteristicDiscoveryCallback, UUID::ShortUUIDBytes_t(BLE_UUID_UNKNOWN), UUID::ShortUUIDBytes_t(BLE_UUID_UNKNOWN));
    }
}

void disconnectionCallback(Gap::Handle_t handle, Gap::DisconnectionReason_t reason) {
    printf("disconnected\r\n");
}

void DataWriteCallback(const GattWriteCallbackParams *params) {
    Gap::Handle_t connectionHandle = params->handle;
    printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %16x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
    printf("\r\n");
    if (command == 1) {
        //ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x12, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x19, 0x05, (reinterpret_cast<const uint8_t *>(&data_setup)));
        command = command + 1;
        //ble.gattClient().processWriteResponse(WriteResponseCallback);
        printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
        printf("\r\n");
    }
    else if (command == 2) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x28, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        command = command + 1;
        printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
        printf("\r\n");
    }
    else if (command == 3) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x2c, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        command = command + 1;
        printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
        printf("\r\n");
    }
    else if (command == 4) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x2f, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        command = command + 1;
        printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
        printf("\r\n");
    }
    else if (command == 5) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x32, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        command = command + 1;
        printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
        printf("\r\n");
    }
    else if (command == 6) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x35, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        command = command + 1;
        printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
           params->writeOp, params->offset, params->len, params->data);
        printf("\r\n");
    }
    else if (command == 7) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x1d, 0x02, (reinterpret_cast<const uint8_t *>(&configure1)));
        command = command + 1;
    }
    else if (command == 8) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x24, 0x02, (reinterpret_cast<const uint8_t *>(&configure2)));
        command = command + 1;
    }
    else if (command == 9) {
        ble.gattClient().write(GattClient::GATT_OP_WRITE_REQ, connectionHandle, 0x19, 0x05, (reinterpret_cast<const uint8_t *>(&data_setup)));
        command = command + 1;
    }
    else {
    printf("V of Firmware = %x ", ble.gattClient().read(connectionHandle, 0x15, 0)); 
    }
}

// void WriteResponseCallback(/*const GattWriteCallbackParams *params*/) {
//    printf("Write result: %02x,\r Offset:  %02x,\r Length: %02x,\r Data: %02x,\r\n",
//           params->writeOp, params->offset, params->len, params->data);
//    printf("\r\n");
//}

/*
void hvxCallback(const GattHVXCallbackParams *params) {
    printf("received HVX callback for handle %u; type %s\r\n", params->handle, (params->type == BLE_HVX_NOTIFICATION) ? "notification" : "indication");
    for (unsigned index = 0; index < params->len; index++) {
        printf(" %02x", params->data[index]);
    }
    printf("\r\n");
}
*/
int main(void)
{
    Ticker ticker;
    ticker.attach(periodicCallback, 1);
    
    
    ble.init();
    ble.gap().onConnection(connectionCallback);
    ble.gap().onDisconnection(disconnectionCallback);

    ble.gap().setScanParams();//Default Values
    ble.gap().startScan(advertisementCallback);
    
    ble.gattClient().onDataWrite(DataWriteCallback);
    //ble.gattClient().processWriteResponse(WriteResponseCallback);
    //ble.gattClient().onHVX(hvxCallback);
    
    while(true) {
        ble.waitForEvent(); // allows or low power operation
    }
}
